@@ -3,7 +3,7 @@ npcdistbw <-
     args = list(...)
     if (is(args[[1]],"formula"))
       UseMethod("npcdistbw",args[[1]])
-    else if (!is.null(args$formula))
+    else if (!is.null(args$formula) && is(args$formula,"formula"))
       UseMethod("npcdistbw",args$formula)
     else
       UseMethod("npcdistbw",args[[which(names(args)=="bws")[1]]])
@@ -11,9 +11,11 @@ npcdistbw <-
 
 npcdistbw.formula <-
   function(formula, data, subset, na.action, call, gdata = NULL, ...){
-    orig.class <- if (missing(data))
-      sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
-    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)),class)
+    orig.class <- tryCatch({
+        if (missing(data))
+            sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
+        else sapply(eval(attr(terms(formula, data=data), "variables"), data, environment(formula)),class)
+    }, error = function(e) "numeric")
 
     has.gval <- !is.null(gdata)
     
@@ -41,7 +43,9 @@ npcdistbw.formula <-
     mf[[1]] <- as.name("model.frame")
     gmf[[1]] <- as.name("model.frame")
 
-    variableNames <- explodeFormula(mf[["formula"]])
+    mf[["formula"]] = eval(mf[[m[1]]], parent.frame())
+    
+    variableNames <- if(!missing(data)) explodeFormula(mf[["formula"]], data = data) else explodeFormula(mf[["formula"]])
     
     ## make formula evaluable, then eval
     varsPlus <- lapply(variableNames, paste, collapse=" + ")
@@ -102,6 +106,9 @@ npcdistbw.condbandwidth <-
            lbc.init = 0.1, hbc.init = 2.0, cfac.init = 0.5, 
            lbd.init = 0.1, hbd.init = 0.9, dfac.init = 0.375, 
            scale.init.categorical.sample=FALSE,
+           transform.bounds = FALSE,
+           invalid.penalty = c("baseline","dbmax"),
+           penalty.multiplier = 10,
            ...){
 
     ydat = toFrame(ydat)
@@ -220,6 +227,9 @@ npcdistbw.condbandwidth <-
     nconfac <- nrow^(-1.0/(2.0*bws$cxkerorder+bws$ncon))
     ncatfac <- nrow^(-2.0/(2.0*bws$cxkerorder+bws$ncon))
 
+    invalid.penalty <- match.arg(invalid.penalty)
+    penalty_mode <- ifelse(invalid.penalty == "baseline", 1L, 0L)
+
     if (bandwidth.compute){
       myopti = list(num_obs_train = nrow,
         num_obs_grid = nog,
@@ -267,7 +277,8 @@ npcdistbw.condbandwidth <-
         cdf_on_train = cdf_on_train,
         int_do_tree = ifelse(options('np.tree'), DO_TREE_YES, DO_TREE_NO),
         scale.init.categorical.sample=scale.init.categorical.sample,
-        dfc.dir = dfc.dir)
+        dfc.dir = dfc.dir,
+        transform.bounds = transform.bounds)
       
       myoptd = list(ftol=ftol, tol=tol, small=small, memfac = memfac,
         lbc.dir = lbc.dir, cfac.dir = cfac.dir, initc.dir = initc.dir, 
@@ -288,8 +299,11 @@ npcdistbw.condbandwidth <-
                bws$ybw[bws$iyuno],bws$ybw[bws$iyord],
                bws$xbw[bws$ixuno],bws$xbw[bws$ixord]),
              fval = double(2),fval.history = double(max(1,nmulti)),
+             eval.history = double(max(1,nmulti)), invalid.history = double(max(1,nmulti)),
              timing = double(1),
-             PACKAGE="npRmpi" )[c("bw","fval","fval.history","timing")])[1]
+             penalty.mode = as.integer(penalty_mode),
+             penalty.multiplier = as.double(penalty.multiplier),
+             PACKAGE="npRmpi" )[c("bw","fval","fval.history","eval.history","invalid.history","timing")])[1]
       } else {
         nbw = double(yncol+xncol)
         gbw = bws$yncon+bws$xncon
@@ -328,6 +342,8 @@ npcdistbw.condbandwidth <-
       tbw$fval = myout$fval[1]
       tbw$ifval = myout$fval[2]
       tbw$fval.history <- myout$fval.history
+      tbw$eval.history <- myout$eval.history
+      tbw$invalid.history <- myout$invalid.history
       tbw$timing <- myout$timing
       tbw$total.time <- total.time
     }
@@ -386,10 +402,12 @@ npcdistbw.condbandwidth <-
                          cykerorder = tbw$cykerorder,
                          uykertype = tbw$uykertype,
                          oykertype = tbw$oykertype,
-                         fval = tbw$fval,
-                         ifval = tbw$ifval,
-                         fval.history = tbw$fval.history,
-                         nobs = tbw$nobs,
+                        fval = tbw$fval,
+                        ifval = tbw$ifval,
+                        fval.history = tbw$fval.history,
+                        eval.history = tbw$eval.history,
+                        invalid.history = tbw$invalid.history,
+                        nobs = tbw$nobs,
                          xdati = tbw$xdati,
                          ydati = tbw$ydati,      
                          xnames = tbw$xnames,
@@ -446,6 +464,7 @@ npcdistbw.default <-
            lbc.init, hbc.init, cfac.init, 
            lbd.init, hbd.init, dfac.init, 
            scale.init.categorical.sample,
+           transform.bounds, invalid.penalty, penalty.multiplier,
            ## dummy arguments for condbandwidth() function call
            bwmethod, bwscaling, bwtype,
            cxkertype, cxkerorder,
@@ -493,7 +512,10 @@ npcdistbw.default <-
                "lbd.dir", "hbd.dir", "dfac.dir", "initd.dir", 
                "lbc.init", "hbc.init", "cfac.init", 
                "lbd.init", "hbd.init", "dfac.init", 
-               "scale.init.categorical.sample")
+               "scale.init.categorical.sample",
+               "transform.bounds",
+               "invalid.penalty",
+               "penalty.multiplier")
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
@@ -508,4 +530,3 @@ npcdistbw.default <-
 
     return(tbw)
   }
-
