@@ -3,7 +3,7 @@ npregbw <-
     args = list(...)
     if (is(args[[1]],"formula"))
       UseMethod("npregbw",args[[1]])
-    else if (!is.null(args$formula))
+    else if (!is.null(args$formula) && is(args$formula,"formula"))
       UseMethod("npregbw",args$formula)
     else
       UseMethod("npregbw",args[[which(names(args)=="bws")[1]]])
@@ -12,9 +12,11 @@ npregbw <-
 npregbw.formula <-
   function(formula, data, subset, na.action, call, ...){
 
-    orig.class <- if (missing(data))
-      sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
-    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)),class)
+    orig.class <- tryCatch({
+        if (missing(data))
+            sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
+        else sapply(eval(attr(terms(formula, data=data), "variables"), data, environment(formula)),class)
+    }, error = function(e) "numeric")
 
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"),
@@ -95,7 +97,11 @@ npregbw.rbandwidth <-
            lbd.dir = 0.1, hbd.dir = 1, dfac.dir = 0.25*(3.0-sqrt(5)), initd.dir = 1.0, 
            lbc.init = 0.1, hbc.init = 2.0, cfac.init = 0.5, 
            lbd.init = 0.1, hbd.init = 0.9, dfac.init = 0.375, 
-           scale.init.categorical.sample = FALSE,...){
+           scale.init.categorical.sample = FALSE,
+           transform.bounds = FALSE,
+           invalid.penalty = c("baseline","dbmax"),
+           penalty.multiplier = 10,
+           ...){
 
     xdat <- toFrame(xdat)
 
@@ -154,6 +160,9 @@ npregbw.rbandwidth <-
     nconfac <- nrow^(-1.0/(2.0*bws$ckerorder+bws$ncon))
     ncatfac <- nrow^(-2.0/(2.0*bws$ckerorder+bws$ncon))
 
+    invalid.penalty <- match.arg(invalid.penalty)
+    penalty_mode <- ifelse(invalid.penalty == "baseline", 1L, 0L)
+
     if (bandwidth.compute){
       myopti = list(num_obs_train = dim(xdat)[1], 
         iMultistart = ifelse(nmulti==0,IMULTI_FALSE,IMULTI_TRUE),
@@ -188,7 +197,8 @@ npregbw.rbandwidth <-
           ll = REGTYPE_LL),
         int_do_tree = ifelse(options('np.tree'), DO_TREE_YES, DO_TREE_NO),
         scale.init.categorical.sample = scale.init.categorical.sample,
-        dfc.dir = dfc.dir)
+        dfc.dir = dfc.dir,
+        transform.bounds = transform.bounds)
       
       myoptd = list(ftol=ftol, tol=tol, small=small,
         lbc.dir = lbc.dir, cfac.dir = cfac.dir, initc.dir = initc.dir, 
@@ -205,8 +215,11 @@ npregbw.rbandwidth <-
            as.integer(myopti), as.double(myoptd), 
            bw = c(bws$bw[bws$icon],bws$bw[bws$iuno],bws$bw[bws$iord]),
            fval = double(2),fval.history = double(max(1,nmulti)),
+           eval.history = double(max(1,nmulti)), invalid.history = double(max(1,nmulti)),
            timing = double(1),
-           PACKAGE="npRmpi" )[c("bw","fval","fval.history","timing")])[1]
+           penalty.mode = as.integer(penalty_mode),
+           penalty.multiplier = as.double(penalty.multiplier),
+           PACKAGE="npRmpi" )[c("bw","fval","fval.history","eval.history","invalid.history","timing")])[1]
       
 
       rorder = numeric(ncol)
@@ -216,6 +229,8 @@ npregbw.rbandwidth <-
       tbw$fval <- myout$fval[1]
       tbw$ifval <- myout$fval[2]
       tbw$fval.history <- myout$fval.history
+      tbw$eval.history <- myout$eval.history
+      tbw$invalid.history <- myout$invalid.history
       tbw$timing <- myout$timing
       tbw$total.time <- total.time
     }
@@ -262,6 +277,8 @@ npregbw.rbandwidth <-
                       fval = tbw$fval,
                       ifval = tbw$ifval,
                       fval.history = tbw$fval.history,
+                      eval.history = tbw$eval.history,
+                      invalid.history = tbw$invalid.history,
                       nobs = tbw$nobs,
                       xdati = tbw$xdati,
                       ydati = tbw$ydati,
@@ -288,8 +305,11 @@ npregbw.default <-
            lbc.dir, dfc.dir, cfac.dir, initc.dir, 
            lbd.dir, hbd.dir, dfac.dir, initd.dir, 
            lbc.init, hbc.init, cfac.init, 
-           lbd.init, hbd.init, dfac.init,
+           lbd.init, hbd.init, dfac.init, 
            scale.init.categorical.sample,
+           transform.bounds = FALSE,
+           invalid.penalty = c("baseline","dbmax"),
+           penalty.multiplier = 10,
            ## dummy arguments for later passing into rbandwidth()
            regtype, bwmethod, bwscaling, bwtype,
            ckertype, ckerorder, ukertype, okertype,
@@ -327,7 +347,10 @@ npregbw.default <-
                "lbd.dir", "hbd.dir", "dfac.dir", "initd.dir", 
                "lbc.init", "hbc.init", "cfac.init", 
                "lbd.init", "hbd.init", "dfac.init", 
-               "scale.init.categorical.sample")
+               "scale.init.categorical.sample",
+               "transform.bounds",
+               "invalid.penalty",
+               "penalty.multiplier")
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
@@ -343,4 +366,3 @@ npregbw.default <-
     return(tbw)
     
   }
-
