@@ -4,18 +4,6 @@
 # dimensionality to a one-dimensional nonparametric estimator, though
 # at the potential cost of misspecification.
 
-# Note - klein and spady's estimator requires a binary y with 0/1
-# values only - TRISTEN XXX kindly check that if someone selects this
-# approach their y is 0/1 and if not return an informative message.
-
-# Note - X must have at least two columns (otherwise foolish to reduce
-# dimensionality) XXX - TRISTEN... kindly check that X has at least
-# two columns.
-
-# Note also that identification requires that X contain at least one
-# continuous variable. XXX - TRISTEN - kindly ensure that this check
-# is made and an appropriate error message is passed back.
-
 # Note also that we will use the so-called scale normalization, i.e.,
 # that beta_1=1 (no need to estimate) which reduces search by 1
 # parameter (this is obviously restricted search subject to beta_1=1).
@@ -25,11 +13,6 @@
 # be 1 by restriction though, at this stage, the user may feed in any
 # value they so desire.
 
-# TRISTEN: I would like this function to accept arguments that
-# npksum will automatically grab... also a pretty print function
-# similar to regression would rock stating `mean in mean, beta in
-# beta' goodness of fit and so forth...
-
 npindex <-
   function(bws, ...){
     args <- list(...)
@@ -53,40 +36,74 @@ npindex <-
   }
 
 npindex.formula <-
-  function(bws, data = NULL, newdata = NULL, ...){
+    function(bws, data = NULL, newdata = NULL, y.eval = FALSE, ...){
 
-    tt <- terms(bws)
-    m <- match(c("formula", "data", "subset", "na.action"),
-               names(bws$call), nomatch = 0)
-    tmf <- bws$call[c(1,m)]
-    tmf[[1]] <- as.name("model.frame")
-    tmf[["formula"]] <- tt
-    umf <- tmf <- eval(tmf, envir = environment(tt))
+        tt <- terms(bws)
+        m <- match(c("formula", "data", "subset", "na.action"),
+                   names(bws$call), nomatch = 0)
+        tmf <- bws$call[c(1,m)]
+        tmf[[1]] <- as.name("model.frame")
+        tmf[["formula"]] <- tt
+        umf <- tmf <- eval(tmf, envir = environment(tt))
 
-    tydat <- model.response(tmf)
-    txdat <- tmf[, attr(attr(tmf, "terms"),"term.labels"), drop = FALSE]
+        tydat <- model.response(tmf)
+        txdat <- tmf[, attr(attr(tmf, "terms"),"term.labels"), drop = FALSE]
 
-    if ((has.eval <- !is.null(newdata))) {
-      if (!(has.ey <- succeedWithResponse(tt, newdata)))
-        tt <- delete.response(tt)
-      
-      umf <- emf <- model.frame(tt, data = newdata)
+        if ((has.eval <- !is.null(newdata))) {
+          if (!y.eval){
+            tt <- delete.response(tt)
 
-      if (has.ey)
-        eydat <- model.response(emf)
-      
-      exdat <- emf[, attr(attr(emf, "terms"),"term.labels"), drop = FALSE]
+            orig.class <- sapply(eval(attr(tt, "variables"), newdata, environment(tt)),class)
+            
+            ## delete.response clobbers predvars, which is used for timeseries objects
+            ## so we need to reconstruct it
+
+            if(all(orig.class == "ts")){
+              args <- (as.list(attr(tt, "variables"))[-1])
+              attr(tt, "predvars") <- as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), args))))
+            }else if(any(orig.class == "ts")){
+              arguments <- (as.list(attr(tt, "variables"))[-1])
+              arguments.normal <- arguments[which(orig.class != "ts")]
+              arguments.timeseries <- arguments[which(orig.class == "ts")]
+
+              ix <- sort(c(which(orig.class == "ts"),which(orig.class != "ts")),index.return = TRUE)$ix
+              attr(tt, "predvars") <- bquote(.(as.call(c(quote(cbind),as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), arguments.timeseries)))),arguments.normal,check.rows = TRUE)))[,.(ix)])
+            }else{
+              attr(tt, "predvars") <- attr(tt, "variables")
+            }
+          }
+          
+          umf <- emf <- model.frame(tt, data = newdata)
+
+          if (y.eval)
+            eydat <- model.response(emf)
+          
+          exdat <- emf[, attr(attr(emf, "terms"),"term.labels"), drop = FALSE]
+        }
+
+        ev <-
+            eval(parse(text=paste("npindex(txdat = txdat, tydat = tydat,",
+                           ifelse(has.eval,paste("exdat = exdat,",ifelse(y.eval,"eydat = eydat,","")),""),
+                           "bws = bws, ...)")))
+
+        ev$omit <- attr(umf,"na.action")
+        ev$rows.omit <- as.vector(ev$omit)
+        ev$nobs.omit <- length(ev$rows.omit)
+
+        ev$mean <- napredict(ev$omit, ev$mean)
+        ev$merr <- napredict(ev$omit, ev$merr)
+
+        if(ev$gradients){
+            ev$grad <- napredict(ev$omit, ev$grad)
+            ev$gerr <- napredict(ev$omit, ev$gerr)
+        }
+
+        if(ev$residuals){
+            ev$resid <- naresid(ev$omit, ev$resid)
+        }    
+        return(ev)
     }
 
-    ev <-
-    eval(parse(text=paste("npindex(txdat = txdat, tydat = tydat,",
-                 ifelse(has.eval,paste("exdat = exdat,",ifelse(has.ey,"eydat = eydat,","")),""),
-                 "bws = bws, ...)")))
-    ev$rows.omit <- as.vector(attr(umf,"na.action"))
-    ev$nobs.omit <- length(ev$rows.omit)
-    ev
-  }
-
 npindex.call <-
   function(bws, ...) {
     npindex(txdat = eval(bws$call[["xdat"]], environment(bws$call)),
@@ -95,7 +112,8 @@ npindex.call <-
   }
 
 npindex.default <- function(bws, txdat, tydat, ...){
-  sc.names <- names(sys.call())
+  sc <- sys.call()
+  sc.names <- names(sc)
 
   ## here we check to see if the function was called with tdat =
   ## if it was, we need to catch that and map it to dat =
@@ -110,45 +128,28 @@ npindex.default <- function(bws, txdat, tydat, ...){
   no.tydat <- missing(tydat)
 
   ## if bws was passed in explicitly, do not compute bandwidths
-    
+
   if(txdat.named)
     txdat <- toFrame(txdat)
 
-  mc <- match.call()
-
-  tx.str <- ifelse(txdat.named, "xdat = txdat,",
-                   ifelse(no.txdat, "", "txdat,"))
-  ty.str <- ifelse(tydat.named, "ydat = tydat,",
-                   ifelse(no.tydat, "", "tydat,"))
+  sc.bw <- sc
   
-  tbw <- eval(parse(text = paste("npindexbw(",
-                      ifelse(bws.named,                             
-                             paste(tx.str, ty.str,
-                                   "bws = bws, bandwidth.compute = FALSE,"),
-                             paste(ifelse(no.bws, "", "bws,"), tx.str, ty.str)),
-                      "call = mc, ...",")",sep="")))
+  sc.bw[[1]] <- quote(npindexbw)
 
-  ##tbw <- updateBwNameMetadata(nameList =
-  ##                            list(ynames = deparse(substitute(tydat))),
-  ##                            bws = tbw)
-
-  repair.args <- c("data", "subset", "na.action")
-  
-  m.par <- match(repair.args, names(mc), nomatch = 0)
-  m.child <- match(repair.args, names(tbw$call), nomatch = 0)
-
-  if(any(m.child > 0)) {
-    tbw$call[m.child] <- mc[m.par]
+  if(bws.named){
+    sc.bw$bandwidth.compute <- FALSE
   }
 
-  ## next we repair arguments portion of the call
-  m.bws.par <- match(c("bws","txdat","tydat"), names(mc), nomatch = 0)
-  m.bws.child <- match(c("bws","txdat","tydat"), as.character(tbw$call), nomatch = 0)
-  m.bws.union <- (m.bws.par > 0) & (m.bws.child > 0)
+  ostxy <- c('txdat','tydat')
+  nstxy <- c('xdat','ydat')
   
-  tbw$call[m.bws.child[m.bws.union]] <- mc[m.bws.par[m.bws.union]]
+  m.txy <- match(ostxy, names(sc.bw), nomatch = 0)
 
-  environment(tbw$call) <- parent.frame()
+  if(any(m.txy > 0)) {
+    names(sc.bw)[m.txy] <- nstxy[m.txy > 0]
+  }
+    
+  tbw <- eval.parent(sc.bw)
 
   ## convention: drop 'bws' and up to two unnamed arguments (including bws)
   if(no.bws){
@@ -156,13 +157,13 @@ npindex.default <- function(bws, txdat, tydat, ...){
     ty.str <- ",tydat = tydat"
   } else {
     tx.str <- ifelse(txdat.named, ",txdat = txdat","")
-    ty.str <- ifelse(tydat.named, ",tydat = tydat","")    
+    ty.str <- ifelse(tydat.named, ",tydat = tydat","")
     if((!bws.named) && (!txdat.named)){
       ty.str <- ifelse(tydat.named, ",tydat = tydat",
                        ifelse(no.tydat,"",",tydat"))
     }
   }
-  
+
   eval(parse(text=paste("npindex(bws = tbw", tx.str, ty.str, ",...)")))
 }
 
@@ -177,17 +178,15 @@ npindex.sibandwidth <-
            errors = FALSE,
            boot.num = 399, ...) {
 
-  
     no.ex = missing(exdat)
     no.ey = missing(eydat)
-    
 
     ## if no.ex then if !no.ey then ey and tx must match, to get
     ## oos errors alternatively if no.ey you get is errors if
     ## !no.ex then if !no.ey then ey and ex must match, to get
     ## oos errors alternatively if no.ey you get NO errors since we
     ## don't evaluate on the training data
-    
+
     txdat = toFrame(txdat)
 
     if (!(is.vector(tydat) | is.factor(tydat)))
@@ -201,15 +200,15 @@ npindex.sibandwidth <-
 
     if (!no.ex){
       exdat = toFrame(exdat)
-      
+
       if (! txdat %~% exdat )
         stop("'txdat' and 'exdat' are not similar data frames!")
-      
+
       if (!no.ey){
         if (dim(exdat)[1] != length(eydat))
           stop("number of evaluation data 'exdat' and dependent data 'eydat' do not match")
       }
-      
+
     } else if(!no.ey) {
       if (dim(txdat)[1] != length(eydat))
         stop("number of training data 'txdat' and dependent data 'eydat' do not match")
@@ -226,7 +225,6 @@ npindex.sibandwidth <-
     txdat = txdat[goodrows,,drop = FALSE]
     tydat = tydat[goodrows]
 
-
     if (!no.ex){
       goodrows = 1:dim(exdat)[1]
       rows.omit = eval(parse(text=paste('attr(na.omit(data.frame(exdat',
@@ -252,7 +250,6 @@ npindex.sibandwidth <-
     else
       tydat <- as.double(tydat)
 
-
     if (no.ey)
       eydat <- double()
     else {
@@ -266,9 +263,9 @@ npindex.sibandwidth <-
 
     ## re-assign levels in training and evaluation data to ensure correct
     ## conversion to numeric type.
-    
+
     txdat <- adjustLevels(txdat, bws$xdati)
-      
+
     if (!no.ex)
       exdat <- adjustLevels(exdat, bws$xdati)
 
@@ -280,25 +277,21 @@ npindex.sibandwidth <-
 
     ## put the unordered, ordered, and continuous data in their own objects
     ## data that is not a factor is continuous.
-    
+
     txdat = toMatrix(txdat)
 
     if (!no.ex){
       exdat = toMatrix(exdat)
     }
 
-
     ## from this point on txdat and exdat have been recast as matrices
 
-    ## TRISTEN - how do we cast txdat here so that matrix multiplication
-    ## is permitted? I currently use cbind() but data.frame barfs out...
-
     ## First, create the scalar index (n \times 1 vector)
 
     index <- txdat %*% bws$beta
 
     if(no.ex) {
-      index.eval <- index 
+      index.eval <- index
       exdat <- txdat
       eydat <- tydat
     } else {
@@ -309,32 +302,31 @@ npindex.sibandwidth <-
 
     if(gradients==FALSE) {
 
-      index.mean <- npksum(txdat=index,
-                           tydat=tydat,
-                           exdat=index.eval,
-                           bws=bws$bw,
-                           ckertype = bws$ckertype,
-                           ckerorder = bws$ckerorder)$ksum/
-                             npksum(txdat=index,
-                                    exdat=index.eval,
-                                    bws=bws$bw,                                    
-                                    ckertype = bws$ckertype,
-                                    ckerorder = bws$ckerorder)$ksum
+      tww <- npksum(txdat=as.matrix(txdat) %*% as.matrix(bws$beta),
+                    tydat=as.matrix(data.frame(tydat,1)),
+                    weights=as.matrix(data.frame(tydat,1)),
+                    exdat=as.matrix(exdat) %*% as.matrix(bws$beta),
+                    bws=bws$bw,
+                    ckertype = bws$ckertype,
+                    ckerorder = bws$ckerorder)$ksum
+
+      index.mean <- tww[1,2,]/NZD(tww[2,2,])
 
       if(!no.ex & (no.ey | residuals)){
+
         ## want to evaluate on training data for in sample errors even
         ## if evaluation x's are different from training but no y's
         ## are specified
-        
-        index.tmean <- npksum(txdat=index,
-                              tydat=tydat,
-                              bws=bws$bw,                              
-                              ckertype = bws$ckertype,
-                              ckerorder = bws$ckerorder)$ksum/
-                                npksum(txdat=index,
-                                       bws=bws$bw,                                       
-                                       ckertype = bws$ckertype,
-                                       ckerorder = bws$ckerorder)$ksum
+
+        tww <- npksum(txdat=as.matrix(txdat) %*% as.matrix(bws$beta),
+                      tydat=as.matrix(data.frame(tydat,1)),
+                      weights=as.matrix(data.frame(tydat,1)),
+                      bws=bws$bw,
+                      ckertype = bws$ckertype,
+                      ckerorder = bws$ckerorder)$ksum
+
+        index.tmean <- tww[1,2,]/NZD(tww[2,2,])
+
       }
 
     } else if(gradients==TRUE) {
@@ -342,18 +334,18 @@ npindex.sibandwidth <-
       model <- npreg(txdat=index,
                      tydat=tydat,
                      exdat=index.eval,
-                     bws=bws$bw,                     
+                     bws=bws$bw,
                      ckertype = bws$ckertype,
                      ckerorder = bws$ckerorder,
                      regtype="lc",
                      gradients=TRUE)
 
       index.mean <- model$mean
-      
+
       ## index.grad is a matrix, one column for each variable, each
       ## equal to its coefficient beta_i times the first derivative of
       ## the local-constant model
-      
+
       index.grad <- as.matrix(model$grad)%*%t(as.vector(bws$beta))
 
       if(!no.ex & (no.ey | residuals)){
@@ -373,11 +365,10 @@ npindex.sibandwidth <-
 
         index.tmean <- model$mean
 
-        index.tgrad <- as.matrix(model$grad)%*%t(as.vector(bws$beta))        
+        index.tgrad <- model$grad
 
       }
 
-
     }
 
     if (no.ex) {
@@ -414,11 +405,12 @@ npindex.sibandwidth <-
       tyindex <- npksum(txdat = index,
                         tydat = rep(1,length(tydat)),
                         weights = W,
-                        bws = bws$bw,                        
+                        bws = bws$bw,
                         ckertype = bws$ckertype,
                         ckerorder = bws$ckerorder)$ksum
 
-      tindex <- npksum(txdat = index, bws = bws$bw,                       
+      tindex <- npksum(txdat = index,
+                       bws = bws$bw,
                        ckertype = bws$ckertype,
                        ckerorder = bws$ckerorder)$ksum
 
@@ -434,31 +426,30 @@ npindex.sibandwidth <-
       ## Need to trap case where k-1=1..., sapply will return a
       ## vector, need a 1 x n matrix
 
-      if(is.vector(xmex)) xmex <- matrix(xmex,nrow=1,ncol=length(xmex))
-
-      ## g^{(1)}=dg/d\beta, first beta normalized to one so this
-      ## simplifies computation (beta's drop out)
-
-      dg.db.sq <- (W*index.tgrad[,1])^2
-
-      dg.db.sq.xmex <- sapply(1:length(tydat),function(i){dg.db.sq[i,]*xmex[,i]})      
+      ## Need to trap case where k-1=1..., sapply will return a
+      ## vector, need a 1 x n matrix
+      if(is.vector(xmex)) {
+        dg.db.xmex <- matrix(index.tgrad[,1]*xmex,nrow=1,ncol=length(xmex))
+      } else {
+        dg.db.xmex <- index.tgrad[,1]*xmex
+      }
 
       uhat <- tydat - index.tmean ## Training y and training mean
 
-      Vinv <- solve(dg.db.sq.xmex%*%t(xmex))
+      Vinv <- chol2inv(chol(dg.db.xmex%*%t(dg.db.xmex)))
 
-      Sigma <- ((uhat^2)*dg.db.sq.xmex)%*%t(xmex)
+      Sigma <- (uhat*dg.db.xmex)%*%t(uhat*dg.db.xmex)
 
       Bvcov[-1,-1] <- Vinv %*% Sigma %*% Vinv
-    
-      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)      
+
+      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)
 
       ## Now export this in an S3 method...
 
     } else if(bws$method == "kleinspady" & gradients == TRUE) {
 
       ## We divide by P(1-P) so test for P=0 or 1...
-      
+
       keep <- which(index.tmean < 1 & index.tmean > 0)
       dg.db <- txdat[,-1,drop=FALSE]*index.tgrad[,1]
 
@@ -469,47 +460,43 @@ npindex.sibandwidth <-
 
       q <- ncol(txdat)
       Bvcov <- matrix(0,q,q)
-      Bvcov[-1,-1] <- solve(t(dg.db[keep,])%*%(dg.db[keep,]/(index.tmean[keep]*
-        (1-index.tmean[keep]))))
+      Bvcov[-1,-1] <- chol2inv(chol(t(dg.db[keep,])%*%(dg.db[keep,]/(index.tmean[keep]*
+        (1-index.tmean[keep])))))
 
-      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)      
+      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)
 
       ## Now export this in an S3 method...
 
     }
 
-    ## TRISTEN XXX - for continuous y we want to return the fitted model
-    ## along with the measures of goodness of fit RSQ, MSE, and other
-    ## measures of goodness of fit. For discrete y (0/1), npconmode()
-    ## type measures. But, I don't want to implement asymptotic standard
-    ## errors however, these can be readily bootstrapped, so perhaps we
-    ## can add a simple bootstrap section that bootstraps beta and the
-    ## mean (and gradients and avgderiv)?
-
     if (gradients){
       boofun = function(data, indices){
-        rindex = txdat[indices,] %*% bws$beta
-        model = npreg(regtype = 'lc', gradients = TRUE,
-          txdat = rindex,
-          tydat = tydat[indices],
-          exdat = index.eval,
-          bws = bws$bw,
-          ckertype = bws$ckertype,
-          ckerorder = bws$ckerorder)[c('mean','grad')]
-
+        rindex <- txdat[indices,] %*% bws$beta
+        model <- npreg(regtype = 'lc',
+                       gradients = TRUE,
+                       txdat = rindex,
+                       tydat = tydat[indices],
+                       exdat = index.eval,
+                       bws = bws$bw,
+                       ckertype = bws$ckertype,
+                       ckerorder = bws$ckerorder)[c('mean','grad')]
+        
         c(model$mean, model$grad, mean(model$grad))
       }
 
     } else {
       boofun = function(data, indices){
         rindex = txdat[indices,] %*% bws$beta
-        npksum(txdat = rindex, tydat = tydat[indices], exdat = index.eval,
-               bws = bws$bw,
-               ckertype = bws$ckertype,
-               ckerorder = bws$ckerorder)$ksum/
-                 npksum(txdat = rindex, exdat = index.eval, bws=bws$bw,                        
-                        ckertype = bws$ckertype,
-                        ckerorder = bws$ckerorder)$ksum
+        tww <- npksum(txdat = rindex,
+                      tydat = cbind(tydat[indices],1),
+                      weights = cbind(tydat[indices],1),
+                      exdat = index.eval,
+                      bws = bws$bw,
+                      ckertype = bws$ckertype,
+                      ckerorder = bws$ckerorder)$ksum
+
+        tww[1,2,]/NZD(tww[2,2,])
+        
       }
     }
 
@@ -524,7 +511,7 @@ npindex.sibandwidth <-
         index.gerr = matrix(data = 0, ncol = ncol(txdat), nrow = length(index.eval))
         index.gerr[,] = sqrt(diag(cov(boot.out$t[,(length(index.eval)+1):(2*length(index.eval))])))
 
-        for (i in ncol(txdat))
+        for (i in 1:ncol(txdat))
           index.gerr[,i] = abs(bws$beta[i])*index.gerr[,i]
 
         index.mgerr = sd(boot.out$t[,2*length(index.eval)+1])
@@ -555,27 +542,24 @@ npindex.sibandwidth <-
       index.pred =
         if (!no.ey) round(index.mean)
         else round(index.tmean)
-      
+
       confusion.matrix =
         table(if (!no.ey) eydat else tydat,
               index.pred, dnn=c("Actual", "Predicted"))
-      
+
       CCR.overall <- sum(diag(confusion.matrix))/sum(confusion.matrix)
       CCR.byoutcome <- diag(confusion.matrix)/rowSums(confusion.matrix)
 
-
       fit.mcfadden <- confusion.matrix/sum(confusion.matrix)
-      
+
       fit.mcfadden <- sum(diag(fit.mcfadden)) -
         (sum(fit.mcfadden^2)-sum(diag(fit.mcfadden)^2))
 
-
       strgof = "confusion.matrix = confusion.matrix, CCR.overall = CCR.overall,
            CCR.byoutcome =  CCR.byoutcome, fit.mcfadden = fit.mcfadden,"
       strres = ""
     }
 
-
     eval(parse(text=paste(
                  "singleindex(bws = bws, index = index.eval, mean = index.mean,",
                  ifelse(errors,"merr = index.merr,",""),
@@ -584,5 +568,5 @@ npindex.sibandwidth <-
                  strres,
                  "ntrain = nrow(txdat),", strgof,
                  "trainiseval = no.ex, residuals = residuals, gradients = gradients)")))
-  
+
   }
