@@ -3,7 +3,7 @@ npscoefbw <-
     args = list(...)
     if (is(args[[1]],"formula"))
       UseMethod("npscoefbw",args[[1]])
-    else if (!is.null(args$formula))
+    else if (!is.null(args$formula) && is(args$formula,"formula"))
       UseMethod("npscoefbw",args$formula)
     else
       UseMethod("npscoefbw",args[[which(names(args)=="bws")[1]]])
@@ -11,9 +11,11 @@ npscoefbw <-
 
 npscoefbw.formula <-
   function(formula, data, subset, na.action, call, ...){
-    orig.class <- if (missing(data))
-      sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
-    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)),class)
+    orig.class <- tryCatch({
+        if (missing(data))
+            sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
+        else sapply(eval(attr(terms(formula, data=data), "variables"), data, environment(formula)),class)
+    }, error = function(e) "numeric")
 
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"),
@@ -23,7 +25,7 @@ npscoefbw.formula <-
     if(!missing(call) && is.call(call)){
       ## rummage about in the call for the original formula
       for(i in 1:length(call)){
-        if(tryCatch(class(eval(call[[i]])) == "formula",
+        if(tryCatch(inherits(eval(call[[i]], parent.frame()), "formula"),
                     error = function(e) FALSE))
           break;
       }
@@ -32,7 +34,8 @@ npscoefbw.formula <-
 
     mf[[1]] <- as.name("model.frame")
 
-    chromoly <- explodePipe(mf[["formula"]])
+    formula_to_explode <- eval(mf[["formula"]], parent.frame())
+    chromoly <- explodePipe(formula_to_explode, env = environment(formula))
 
     bronze <- sapply(chromoly, paste, collapse = " + ")
     mf[["formula"]] <-
@@ -89,7 +92,6 @@ npscoefbw.NULL <-
            bws, ...){
 
     miss.z <- missing(zdat)
-    ##class(xdat)
 
     xdat <- toFrame(xdat)
 
@@ -156,7 +158,7 @@ npscoefbw.scbandwidth <-
     }
 
     if (!(is.vector(ydat) | is.factor(ydat)))
-      stop("'ydat' must be a vector")
+      stop("'ydat' must be a vector or a factor")
 
     eval(parse(text = paste("bwMatch(",
                  ifelse(miss.z, "xdat, bws$xdati", "zdat, bws$zdati"),")")))
@@ -589,7 +591,7 @@ npscoefbw.default <-
     xdat <- toFrame(xdat)
     
     if(!(is.vector(ydat) | is.factor(ydat)))
-      stop("'ydat' must be a vector")
+      stop("'ydat' must be a vector or a factor")
 
     if(!miss.z)
       zdat <- toFrame(zdat)
