@@ -18,7 +18,12 @@ mpi.bcast.cmd(options(np.messages=FALSE),
              caller.execute=TRUE)
 
 data("Italy")
-n <- nrow(Italy)
+
+
+
+n <- 1008 # n <- nrow(Italy)
+
+
 mpi.bcast.Robj2slave(Italy)
 
 ## A quantile regression example
