@@ -23,10 +23,12 @@ extern  int iSeed_my_rank;
 extern  MPI_Status status;
 #endif
 
-#ifdef RCSID
-static char rcsid[] = "$Id: kernel.c,v 1.3 2006/11/02 16:56:49 tristen Exp $";
-#endif
+#include <R.h>
 
+extern double np_tgauss2_b, np_tgauss2_alpha, np_tgauss2_c0;
+// convolution kernel constants
+extern double np_tgauss2_a0, np_tgauss2_a1, np_tgauss2_a2;
+extern double np_tgauss2_k, np_tgauss2_k2, np_tgauss2_k22, np_tgauss2_km;
 /*
 
 The following kernel functions are supported.
@@ -48,6 +50,7 @@ convergence rate of the bandwidth.
 
 8 = Rectangular kernel
 
+9 = Truncated Gaussian kernel (second order)
 The following techniques are currently supported.
 
 The technique is input via disk files bandwidth_d.ini and bandwidth_r.ini
@@ -161,8 +164,12 @@ double kernel(int KERNEL, double z)
 
 			break;
 
+  case 9:
+    return_value = (fabs(z) > np_tgauss2_b) ? 0.0 : np_tgauss2_alpha*ONE_OVER_SQRT_TWO_PI*exp(-0.5*z*z) - np_tgauss2_c0;
+    break;
 	}
 
+
 	return(return_value);
 
 }
@@ -321,6 +328,9 @@ double cdf_kernel(int KERNEL, double z)
 
 			break;
 
+  case 9:
+    return_value = (z <= -np_tgauss2_b) ? 0.0 : ((z >= np_tgauss2_b) ? 1.0 : (np_tgauss2_alpha*0.5*erfun(0.7071067810*z)-np_tgauss2_c0*z + 0.5));
+    break;
 	}
 
 	return(return_value);
@@ -447,6 +457,9 @@ double kernel_deriv(int KERNEL, double z)
 
 			break;
 
+  case 9:
+    return_value = (fabs(z) > np_tgauss2_b) ? 0.0 : np_tgauss2_alpha*(-z*ONE_OVER_SQRT_TWO_PI*exp(-0.5*z*z));
+    break;
 	}
 
 	return(return_value);
@@ -520,7 +533,7 @@ double kernel_convol(int KERNEL, int BANDWIDTH, double z, double h1, double h2)
 
 /* Second Order Epanechnikov */
 
-/* Error pointed out by Alicia Prez-Alonso and colleagues May 22
+/* Error pointed out by Alicia Perez-Alonso and colleagues May 22
    2009, previous version incorporated incorrect limits of
    integration. Confirmed May 27 and incorporated. New version
    requires additional test for negative and positive values, and
@@ -586,6 +599,20 @@ double kernel_convol(int KERNEL, int BANDWIDTH, double z, double h1, double h2)
 
 				break;
 
+    case 9:
+      if(fabs(z) > 2*np_tgauss2_b)
+        return_value = 0.0;
+      else {
+        if(z < 0)
+          return_value = (np_tgauss2_a0*erfun(0.5*z + np_tgauss2_b)*exp(-0.25*z*z) + np_tgauss2_a1*z + 
+                          np_tgauss2_a2*erfun(0.7071067810*(z + np_tgauss2_b)) - np_tgauss2_c0);
+        else
+          return_value = (-np_tgauss2_a0*erfun(0.5*z - np_tgauss2_b)*exp(-0.25*z*z) - np_tgauss2_a1*z -
+                          np_tgauss2_a2*erfun(0.7071067810*(z - np_tgauss2_b)) - np_tgauss2_c0);
+        
+      }
+      break;
+
 		}
 
 	}
@@ -840,7 +867,9 @@ double kernel_convol(int KERNEL, int BANDWIDTH, double z, double h1, double h2)
 				}
 
 				break;
-
+    case 9:
+      error("adaptive bandwidths not supported with truncation gaussian."); break;
+      
 		}
 
 	}
@@ -871,7 +900,7 @@ double *DIFF_KER_PPM)
 			*INT_KERNEL_P = 0.28209479177387814348;
 			*K_INT_KERNEL_P = ipow(*INT_KERNEL_P, num_reg_continuous);
 			*INT_KERNEL_PM_HALF = 0.21969564473386119853;
-			*DIFF_KER_PPM = (2.0 * (*K_INT_KERNEL_P/ *INT_KERNEL_P) * 0.06239914704001694495);
+			*DIFF_KER_PPM = (2.0 * (*K_INT_KERNEL_P/ *INT_KERNEL_P) * 0.062399147040017);
 
 			break;
 
@@ -915,7 +944,7 @@ double *DIFF_KER_PPM)
 			*INT_KERNEL_P = 0.26832815729997476357;
 			*K_INT_KERNEL_P = ipow(*INT_KERNEL_P, num_reg_continuous);
 			*INT_KERNEL_PM_HALF = 0.20250390621232470438;
-			*DIFF_KER_PPM = (2.0 * (*K_INT_KERNEL_P/ *INT_KERNEL_P) * 0.06582425108765005919);
+			*DIFF_KER_PPM = (2.0 * (*K_INT_KERNEL_P/ *INT_KERNEL_P) * 0.0658242510876501);
 
 			break;
 
@@ -961,6 +990,15 @@ double *DIFF_KER_PPM)
 
 			break;
 
+    case 9:
+
+			*INT_KERNEL_P = np_tgauss2_k;
+			*K_INT_KERNEL_P = ipow(*INT_KERNEL_P, num_reg_continuous);
+			*INT_KERNEL_PM_HALF = np_tgauss2_km;
+			*DIFF_KER_PPM = 2.0* (*K_INT_KERNEL_P/ *INT_KERNEL_P) * (np_tgauss2_k - np_tgauss2_km);
+
+      break;
+
 	}
 
 	return(0);
@@ -1059,6 +1097,11 @@ double *K_INT_KERNEL_P)
 
 			break;
 
+  case 9:
+    *INT_KERNEL_P = np_tgauss2_k;
+    *K_INT_KERNEL_P = ipow(*INT_KERNEL_P, num_var_continuous);
+    break;
+
 	}
 
 	return(0);
