@@ -114,14 +114,11 @@ extern double **matrix_XY_ordered_eval_extern;
 // cdf extern
 extern double dbl_memfac_ccdf_extern;
 extern double dbl_memfac_dls_extern;
+extern int cdfontrain_extern;
 
 // timing
 extern double timing_extern;
 
-#ifdef RCSID
-static char rcsid[] = "$Id: kernelcv.c,v 1.9 2006/11/02 16:56:49 tristen Exp $";
-#endif
-
 #define RBWM_CVAIC 0
 #define RBWM_CVLS 1
 
@@ -428,6 +425,7 @@ double cv_func_con_distribution_categorical_ls(double *vector_scale_factor)
                                                                            num_reg_unordered_extern,
                                                                            num_reg_ordered_extern,
                                                                            num_reg_continuous_extern,
+                                                                           cdfontrain_extern,
                                                                            dbl_memfac_ccdf_extern,
                                                                            matrix_Y_unordered_train_extern,
                                                                            matrix_Y_ordered_train_extern,
@@ -1050,6 +1048,7 @@ double cv_func_distribution_categorical_ls(double *vector_scale_factor)
                                              num_reg_unordered_extern,
                                              num_reg_ordered_extern,
                                              num_reg_continuous_extern,
+                                             cdfontrain_extern,
                                              dbl_memfac_dls_extern,
                                              matrix_X_unordered_train_extern,
                                              matrix_X_ordered_train_extern,
